/* SR3C, a symbol ranking data compressor.
 *
 * A simple compressor from stdin to stdout, also the one that should
 * build without trouble on a Windows/DOS.
 *
 * Author: Kenneth Oksanen <cessu@iki.fi>, 2008.
 * Copyright (C) Helsinki University of Technology.
 *
 * Driver program sr3cw.c for Windows written by Matt Mahoney.
 *
 * This file is distributed under the following license:

The MIT License
Copyright (c) 2008 Helsinki University of Technology
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

 */


#include <stdio.h>
#include <stdlib.h>
#include "sr3c.h"

static FILE *out;

static int write_f(const unsigned char *bytes, size_t n_bytes, 
		   int flush, void *ctx)
{
  return fwrite(bytes, 1, n_bytes, out) == n_bytes ? 0 : -1;
}


int main(int argc, char **argv)
{
  sr3c_context_t *ctx;
  size_t n;
  unsigned char buf[10000];
  FILE *in;

  /* Check for 3 arguments: c|d input output */
  if (argc < 3 || (argv[1][0]!='c' && argv[1][0]!='d')) {
    printf("To compress|decompress: sr3cw c|d input output\n");
    return 1;
  }

  /* Open input and output files, exit if error */
  in = fopen(argv[2], "rb");
  if (!in)
    perror(argv[2]), exit(1);
  out = fopen(argv[3], "wb");
  if (!out)
    perror(argv[3]), exit(1);

  ctx = sr3c_alloc(write_f, NULL);

  if (argv[1][0] == 'c') {  /* compress */
    while (!feof(in) && !ferror(in)) {
      n = fread(buf, 1, sizeof(buf), in);
      sr3c_compress(buf, n, ctx);
    }
    sr3c_flush(ctx);
  }
  else {  /* decompress */
    while (!feof(in) && !ferror(in)) {
      n = fread(buf, 1, sizeof(buf), in);
      sr3c_uncompress(buf, n, ctx);
    }
  }

  sr3c_free(ctx);
  return 0;
}
